#!/perl5/bin/perl
# ^ ^ ^ ^ ^ --- be sure to change this line to match your server !!!

# DispCSV.pl - Display a Comma-Separated Value (CSV) file, sorted, in a
#              HTML document format.
# Written by:
#   James Greene Informatics Consulting
#   Am Sonnenrain 56
#   D-79539 Lrrach, Germany
#   <greene@gucc.org>
#
# This script is copyright (c) 1997 James Greene Informatics Consulting,
#  Am Sonnenrain 56, D-79539 Lrrach, Germany - <greene@gucc.org>.
#
#                           ----- TERMS OF USE -----
#
# You may copy, use, distribute and modify this script under the condition
# that this copyright message and author's address remain with the modified
# version.
#
# If this script is used on a commercial web site, please calculate how long
# it would have taken you to develop this script youself, multiply this
# value by some reasonable factor, (let's say US$ 25.00 per hour), and
# send that amount to the author at the address above. 
#
# Alternatively, if you have some other (paying) programming tasks,
# please contact me (Jim Greene) to have them done for a reasonable
# cost.
#                           ----- END OF TERMS -----
#
# Version History:
#   V1.6 - 8 April 1998
#       Replaced the SORT function based upon the recommendations in the
#       PERL FAQ (hopefully this will be significantly faster).
#   V1.5 - 18 March 1998
#       General improvements, including strict variable declarations
#       and an attempt to make the whole thing a bit more modular.
#   V1.4 - 04 September 1997
#       Modification to move blank fields to the end of the list when sorting
#   V1.3 - 28 August 1997 (afternoon)
#       Several minor changes after compiling with Perl 5.003_07 with -w switch.
#   V1.2 - 28 August 1997 (morning)
#       The 'sort' column is now printed as column 1 in the output table.
#   V1.1 - 27 August 1997
#       Added the passing of the fieldnames from the calling form
#   V1.0 - 26 August 1997
#       Original script written to the specifications of Bruce Buckner
#       <buckner@halcyon.com>
#
# Command Line (for testing purposes):
# perl -w dispcsv.pl "fieldnames=Name,Car,Wife,Child&fields=Wife"

# use strict ;
# use Benchmark ;
require 5.001 ;

# Set this value to 1 to help debugging the script, 0 for productive use.
# Setting it to 127 will allow the script to be run from the command line,
# rather than as a CGI script.

my $DEBUG = 0 ;

if ($DEBUG) {
    &HTMLHeader("Debug Output Enabled!") ;
}

# Program flow:
#   0.   Set program variables
#   1.   Get command line parameters
#   1.5. Set the rest of the program variables
#   2.   Read CSV file
#   3.   Sort data
#   4.   Display results

#-----------------------------------------------------------------------------
# -0- Set program variables
#-----------------------------------------------------------------------------

# Variable declarations
my (@fieldNames, $dataLocation, $defaultCSV, $defaultSort, $numFields) ;

# 		        --> YOU MAY HAVE TO MAKE SOME CHANGES HERE <--

# Set this value to the directory where the .CSV files are stored - don't
# forget the trailing slash!
$dataLocation   = "/www/buckner/" ;
# $dataLocation   = "/Temp/NTPerl/DISPCSV/" if ($DEBUG == 127) ;


# Set this variable to the name of the default .CSV file (in case you don't set
# the dbname hidden field in your HTML file)
$defaultCSV     = "buckner.csv" ;

# 		        --> YOU DON'T NEED TO CHANGE ANYTHING ELSE HERE <--

#-----------------------------------------------------------------------------
# -1- Get command line parameters
#-----------------------------------------------------------------------------

my (%ARG) ;
&ReadForm(\%ARG) ;

# -- At this point, each field name/value pair is stored in the list $ARG
# Expected Arguments:
#   fields= name of the field to sort by (blank = no sorting)
#   dbname= name of the CSV file to be read (blank = use default)

$ARG{'dbname'} = $defaultCSV if (!defined($ARG{'dbname'})) ;

my $CSVFile = $dataLocation . $ARG{'dbname'} ;

#-----------------------------------------------------------------------------
# -1.5- Set the rest of the program variables
#-----------------------------------------------------------------------------

if (!defined($ARG{'fieldnames'})) {
    HTMLDie("Form Error", "The HTML form calling this script has not defined the required 'fieldnames' parameter!") ;
}

@fieldNames = split /\,/, $ARG{'fieldnames'} ;
$numFields = $#fieldNames + 1 ;

$ARG{'fields'} = $fieldNames[0] if (!defined($ARG{'fields'})) ;

#-----------------------------------------------------------------------------
# -2- Read CSV File
#-----------------------------------------------------------------------------

# +++ April 1998
# Some additional logic was necessary here to eliminate empty lines from
# cluttering up the @CSVData array. This used to read simply
#   open(CSV, ...
#   @CSVData = <CSV> ;
#   close(CSV) ;
# but the new sort logic (using the map command) generated warnings about
# uninitialised values (when running with the -w option).
# --- 

my @CSVData ;
open(CSV,"<$CSVFile") || &HTMLDie("Can't read $CSVFile: $!", "It was not possible to open the file $CSVFile for reading!") ;
while (<CSV>) {
    chomp ;
    next if ($_ eq '') ;
    push @CSVData, ($_) ;
}
close(CSV) ;

#-----------------------------------------------------------------------------
# -3- Sort data
#-----------------------------------------------------------------------------

my ($sortID, @Sorted) ;

# Get the number of the sort field
SortID:
for ($sortID=0 ; $sortID < $numFields ; $sortID++) {
    if ($fieldNames[$sortID] eq $ARG{'fields'}) {
        last SortID ;
    }
}

print "Sorting by field $sortID: $fieldNames[$sortID]\n" if ($DEBUG) ;

# +++ April 1998
#   Big change here - rather than calling a sub for each sort iteration
#   (N log N times), the FAQ recommended to use this code. Hey, I'm always
#   willing to try something new ...
#   This used to read
#       @Sorted = sort SortArray @CSVData ;
#   where sub SortArray would break up the two values, compare the desired
#   column values, and return the cmp result.
# ---

@Sorted = map $_->[0], sort { $a->[1] cmp $b->[1] }
    map { [ $_, (split /\,/)[$sortID] ] } @CSVData;

#-----------------------------------------------------------------------------
# -4- Display results
#-----------------------------------------------------------------------------

# /// display the sort column as the first column \\\
my ($i, $sTemp, @dispOrder) ;

$sTemp = "$sortID " ;
for ($i=0 ; $i < $numFields ; $i++) {
    if ($i != $sortID) {
        $sTemp .= "$i " ;
    }
}

@dispOrder = split / /, $sTemp ;

# 		--> YOU'LL WANT TO MAKE SOME CHANGES HERE <--

&HTMLHeader("Database Results") ;

print <<TX1;

<h1>Database Request</h1>
<p>Sorting database by $fieldNames[$sortID] ... Please wait ... </p>
<div><table border=1>
  <tr>
TX1

for ($i=0 ; $i < $numFields ; $i++) {
    print "    <td><b>$fieldNames[$dispOrder[$i]]</b></td>\n" ;
}

print "  </tr>\n" ;

DISPLOOP:
foreach (@Sorted) {
    my @Temp = split /\,/ ;

# don't display an empty row if there is no data
    next DISPLOOP if ($#Temp == 0) ;

    print "  <tr>\n" ;
    for ($i=0 ; $i < $numFields ; $i++) {
        print "    <td>$Temp[ $dispOrder[$i] ]</td>\n" ;
    }
    print "  </tr>\n" ;
}
print "</table></div>" ;

&HTMLFooter ;

exit 1 ;

#-----------------------------------------------------------------------------
# DONE!
#-----------------------------------------------------------------------------
# Oh yea, the Subroutines...
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# ReadForm - Read the input and put it into the named array that is
#   passed as an argument.
# Parameter:
#   *FORM (OUTPUT) pointer to the named array
# Returns the number of form elements.
# Call with: &ReadForm(\%FORM)
 
sub ReadForm {
    my ($meth, $buffer, $cnt, @pairs) ;
    local (%::FV) ;
    *::FV = shift(@_) ;

    # Get the input
    $meth = (defined($ENV{'REQUEST_METHOD'}) ? $ENV{'REQUEST_METHOD'} : '') ;

    if (!defined($meth) || ($meth eq '') || ($meth eq 'GET')) {
        $buffer = (defined($ENV{'QUERY_STRING'}) ? $ENV{'QUERY_STRING'} : '') ;
        $buffer .= '&' . join('&', @ARGV) ;
    } elsif ($meth eq 'POST') {
        my $len = $ENV{'CONTENT_LENGTH'} ;
        read(STDIN, $buffer, $len) ;
    } else {
        &HTMLDie("Unknown Request Method", "The REQUEST_METHOD '$meth' is not recognised.") ;
    }

    $buffer .= "&fields=Wife&fieldnames=Name,Car,Wife,Child&dbname=buckner.csv" if ($DEBUG == 2) ;

    if ($DEBUG) { 
        print "REQUEST_METHOD = '$meth'<P>\n" ;
        print "\$buffer = $buffer<P>\n" ;
    }
    
    # Split the name-value pairs 
    @pairs = split(/&/, $buffer); 

    $cnt = 0 ;

    PAIRLOOP:
    foreach (@pairs) {
        my ($name, $value) = split /=/ ;
        next PAIRLOOP if (!defined($name) || !defined($value)) ;
 
        # Un-Webify plus signs and %-encoding  
        $name =~ tr/+/ /; 
        $name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg; 
        $value =~ tr/+/ /; 
        $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg; 
 
        # Stop people from using subshells to execute commands 
        # Not a big deal when using sendmail, but very important  
        # when using UCB mail (aka mailx). 
        # $value =~ s/~!/ ~!/g; 
 
        # Debugging 
        print "-- Setting $name to $value --<BR>\n" if ($DEBUG) ;
 
        $::FV{$name} = $value ;
        $cnt++ ;
    }

    return $cnt ;
}

#-----------------------------------------------------------------------------
# HTMLHeader - Print a HTML header
#   Call with: &HTMLHeader($PageTitle) ;
 
sub HTMLHeader
{
    my $Title = shift ;
    my $script = __FILE__ ;

# +++ April 1998
#   Another FAQ-recommended improvement - don't buffer output from CGI scripts.
# ---
    
    $|=1;       # don't buffer output
    
    print <<EOHEAD;
Content-Type: text/html

<html><head>
<title>$Title</title>
<meta name="author" content="James Greene Informatics Consuting">
<meta name="generator" content="$script">
</head>
<body bgcolor="#c0c0c0">
EOHEAD
}

#-----------------------------------------------------------------------------
# DateTime - Print the local date and time (US format)
#   Call with: &DateTime(@lta) ;
# where @lta is a time array (as returned by the localtime() function)

sub DateTime {
    my (@Month, @WDay, @lta, $hour, $apm) ;
    
    @Month = ("January","February","March","April","May","June",
              "July","August","September","October","November","December") ;

    @WDay = ("Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday") ;

    @lta = @_ ;

    $hour = $lta[2] ;
    $apm = ($hour < 12) ? "am" : "pm" ;
    $hour -= 12 if ($hour > 12) ;
    $hour = 12  if (!$hour) ;

    return sprintf("%s, %s %d, %d at %2d:%02d %s",
        $WDay[$lta[6]], $Month[$lta[4]], $lta[3], $lta[5]+1900, $hour, $lta[1], $apm) ;
}


#-----------------------------------------------------------------------------
# HTMLFooter - Print the footer information on an HTML page
#   Call with: &HTMLFooter ;
#
# Go ahead, add whatever buttons or signature information you would like to
# have displayed at the bottom of your page. I would be happy if you would
# leave some note that the script was written by
#   James Greene Informatics Consuting, Am Sonnenrain 56, 79539 Lrrach, Germany
#   mailto:greene@gucc.org

sub HTMLFooter {
    my (@scr, @now, $date, $script, $year) ;
    
    @now = localtime(time) ;
    $date = DateTime(@now) ;
    $year = $now[5] + 1900 ;

    @scr = split /\//, __FILE__ ;
    $script = $scr[$#scr] ;


# 		--> YOU MAY WANT TO MAKE SOME CHANGES HERE <--
    print <<EOFOOT;
<hr>
<div align="left">
<font size=1 face="Verdana">This page generated
<strong>$date</strong> by <strong>$script</strong><br>
Copyright &copy; 1997-1998 <A HREF="http://www.gucc.org/greene/consult">James Greene Informatics Consulting</a>, L&ouml;rrach, Germany<br>
</font><p>
</div>
</body>
</html>
EOFOOT

# 		--> YOU DON'T NEED TO CHANGE ANYTHING ELSE <--
}

#-----------------------------------------------------------------------------
# HtmlDie - Print an HTML formatted error message and terminate.
#   Call with: &HTMLDie($errorHeading, $errorText) ;
sub HTMLDie {
    &HTMLHeader ("Error") if (!$DEBUG) ;
    print "<h1>Error: $_[0]</h1>\n" ;
    print "<b>$_[1]</b>\n" ;
    &HTMLFooter ;
    exit ;
} 

__END__
#-----------------------------------------------------------------------------
# end of file
#-----------------------------------------------------------------------------

